/*
 * Copyright (c) 2010 Adrian Chadd, Xenion Pty Ltd
 * Copyright (c) 2008 Sam Leffler, Errno Consulting
 * Copyright (c) 2008 Atheros Communications, Inc.
 *
 * Permission to use, copy, modify, and/or distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
 * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
 * ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 *
 * $FreeBSD$
 */
#include "opt_ah.h"

#include "ah.h"
#include "ah_internal.h"
#include "ah_devid.h"

#include "ar5416/ar5416.h"
#include "ar5416/ar5416reg.h"
#include "ar5416/ar5416phy.h"

#include "ar9001/ar9160_hwops.h"
#include "ar9001/ar9100_hwops.h"

static uint32_t
ar9100_hw_compute_pll_control(struct ath_hal *ah, const struct ieee80211_channel *chan)
{
	if (chan == AH_NULL)
		return 0x0;		/* XXX ?? */
	if (IEEE80211_IS_CHAN_5GHZ(chan))
		return 0x1450;
	return 0x1458;
}

/* 
 * The AR9100 is based on the AR5416(/AR9160 ?)
 * Some of the parameters are shared.
 */
void
ar9100_hwops_setup(struct ath_hal *ah)
{
        /* Setup AR9160 specific data */
        ar9160_hwops_setup(ah);

	/* Override as needed */
	AH5416(ah)->ah_compute_pll_control = ar9100_hw_compute_pll_control;
}
