
#include "opt_ah.h"

#include "ah.h"
#include "ah_internal.h"
#include "ah_devid.h"

#include "ar5416/ar5416.h"
#include "ar5416/ar5416phy.h"

#include "ar9002/ar9280.h"
#include "ar9002/ar9280_phy.h"
#include "ar9002/ar9280_cal.h"

void
ar9280_hw_olc_temp_compensation(struct ath_hal *ah)
{
	uint32_t rddata, i;
	int delta, currPDADC, regval;
	struct ath_hal_9280 *ahp = AH9280(ah);
	uint8_t hpwr;
	HAL_STATUS es;

	rddata = OS_REG_READ(ah, AR_PHY_TX_PWRCTRL4);
	currPDADC = MS(rddata, AR_PHY_TX_PWRCTRL_PD_AVG_OUT);

	if (ahp->initPDADC == 0 || currPDADC == 0)
		return;

	es = ath_hal_eepromGet(ah, AR_EEP_DAC_HPWR_5G, &hpwr);
	if (es != HAL_OK || hpwr == 0)
		delta = (currPDADC - ahp->initPDADC + 4) / 8;
	else
		delta = (currPDADC - ahp->initPDADC + 5) / 10;

	if (delta != ahp->PDADCdelta) {
		ahp->PDADCdelta = delta;
		for (i = 1; i < AR9280_TX_GAIN_TABLE_SIZE; i++) {
			regval = ahp->originalGain[i] - delta;
			if (regval < 0)
				regval = 0;

			OS_REG_RMW_FIELD(ah,
				      AR_PHY_TX_GAIN_TBL1 + i * 4,
				      AR_PHY_TX_GAIN, regval);
		}
	}
}
