/*
 * Copyright (c) 2010 Adrian Chadd, Xenion Pty Ltd
 * Copyright (c) 2008 Sam Leffler, Errno Consulting
 * Copyright (c) 2008 Atheros Communications, Inc.
 *
 * Permission to use, copy, modify, and/or distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
 * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
 * ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 *
 * $FreeBSD$
 */
#include "opt_ah.h"

#include "ah.h"
#include "ah_internal.h"
#include "ah_devid.h"

#include "ar5416/ar5416.h"
#include "ar5416/ar5416reg.h"

#include "ar9002/ar9280_phy.h"
#include "ar9002/ar9287_phy.h"

#include "ar9002/ar9280_hwops.h"
#include "ar9002/ar9287_hwops.h"

static void
ar9287_hw_set_nf_limits(struct ath_hal *ah)
{
	AH5416(ah)->nf_2g.max = AR_PHY_CCA_MAX_GOOD_VAL_9287_2GHZ;
	AH5416(ah)->nf_2g.min = AR_PHY_CCA_MIN_GOOD_VAL_9280_2GHZ;
	AH5416(ah)->nf_2g.nominal = AR_PHY_CCA_NOM_VAL_9280_2GHZ;
}

/*
 * Involved in temperature calibration - not yet written/ported!
 * - adrian
 */
static void
ar9287_hw_olc_init(struct ath_hal *ah)
{
#ifdef	NOTYET_ADRIAN
	if (! OLC_FOR_AR9287_20_LATER)
		return;

	REG_SET_BIT(ah, AR_PHY_TX_PWRCTRL9,
	    AR_PHY_TX_PWRCTRL9_RES_DC_REMOVAL);
	ath9k_hw_analog_shift_rmw(ah, AR9287_AN_TXPC0,
	    AR9287_AN_TXPC0_TXPCMODE,
	    AR9287_AN_TXPC0_TXPCMODE_S,
	    AR9287_AN_TXPC0_TXPCMODE_TEMPSENSE);
	udelay(100);
#endif
}

void
ar9287_hwops_setup(struct ath_hal *ah)
{
	ar9280_hwops_setup(ah);
	AH5416(ah)->ah_set_nf_limits = ar9287_hw_set_nf_limits;
	AH5416(ah)->ah_olc_init = ar9287_hw_olc_init;
	ar9287_hw_set_nf_limits(ah);
}
